@@ -6,8 +6,8 @@
 ;; Maintainer: Daniel Nouri <daniel.nouri@gmail.com>
 ;; URL: https://github.com/dnouri/pi-coding-agent
 ;; Keywords: ai llm ai-pair-programming tools
-;; Package-Version: 1.3.2
-;; Package-Revision: v1.3.2-0-g88e20cd11f5e
+;; Package-Version: 1.3.6
+;; Package-Revision: v1.3.6-0-g13c5b10f2813
 ;; Package-Requires: ((emacs "28.1") (markdown-mode "2.6") (transient "0.9.0"))
 
 ;; SPDX-License-Identifier: GPL-3.0-or-later
@@ -34,7 +34,7 @@
 ;; with rendered markdown, and a separate prompt composition buffer.
 ;;
 ;; Requirements:
-;;   - pi coding agent 0.51.3 or later, installed and in PATH
+;;   - pi coding agent 0.52.9 or later, installed and in PATH
 ;;
 ;; Optional Dependencies:
 ;;   - phscroll: Markdown tables that exceed the window width wrap awkwardly.
@@ -42,8 +42,11 @@
 ;;     Install from: https://github.com/misohena/phscroll
 ;;
 ;; Usage:
-;;   M-x pi           Start a session in current project
-;;   C-u M-x pi       Start a named session
+;;   M-x pi-coding-agent         Start or focus session in current project
+;;   C-u M-x pi-coding-agent     Start a named session
+;;   M-x pi-coding-agent-toggle  Hide/show session windows in current frame
+;;
+;; Many users define an alias: (defalias 'pi 'pi-coding-agent)
 ;;
 ;; Key Bindings:
 ;;   Input buffer:
@@ -90,6 +93,7 @@ Returns the chat buffer."
          (new-session nil))
     ;; Link buffers to each other
     (with-current-buffer chat-buf
+      (setq default-directory dir)
       (pi-coding-agent--set-input-buffer input-buf)
       ;; Start process if not already running
       (unless (and pi-coding-agent--process (process-live-p pi-coding-agent--process))
@@ -124,6 +128,7 @@ Returns the chat buffer."
       (when new-session
         (pi-coding-agent--display-startup-header)))
     (with-current-buffer input-buf
+      (setq default-directory dir)
       (pi-coding-agent--set-chat-buffer chat-buf))
     chat-buf))
 
@@ -131,7 +136,9 @@ Returns the chat buffer."
 (defun pi-coding-agent (&optional session)
   "Start or switch to pi coding agent session in current project.
 With prefix arg, prompt for SESSION name to allow multiple sessions.
-If already in a pi buffer and no SESSION specified, redisplays current session."
+If already in a pi buffer and no SESSION specified, ensures this session
+is visible. When both chat and input are already shown in the current
+frame, keeps layout unchanged and focuses the input window."
   (interactive
    (list (when current-prefix-arg
            (read-string "Session name: "))))
@@ -146,8 +153,38 @@ If already in a pi buffer and no SESSION specified, redisplays current session."
       (let ((dir (pi-coding-agent--session-directory)))
         (setq chat-buf (pi-coding-agent--setup-session dir session))
         (setq input-buf (buffer-local-value 'pi-coding-agent--input-buffer chat-buf))))
-    ;; Display and focus
-    (pi-coding-agent--display-buffers chat-buf input-buf)))
+    ;; When both windows are already visible in current frame, just focus
+    ;; the session input window. Otherwise restore/show the session layout.
+    (if (and (get-buffer-window-list chat-buf nil)
+             (get-buffer-window-list input-buf nil))
+        (pi-coding-agent--focus-input-window chat-buf input-buf)
+      (pi-coding-agent--display-buffers chat-buf input-buf))))
+
+;;;###autoload
+(defun pi-coding-agent-toggle ()
+  "Toggle pi coding agent window visibility for the current project.
+If pi windows are visible in the current frame, hide them.
+If hidden there but a session exists, show them.
+If no session exists, signal an error."
+  (interactive)
+  (pi-coding-agent--check-dependencies)
+  (let* ((chat-buf (if (derived-mode-p 'pi-coding-agent-chat-mode 'pi-coding-agent-input-mode)
+                       (pi-coding-agent--get-chat-buffer)
+                     (car (pi-coding-agent-project-buffers))))
+         (input-buf (and chat-buf
+                         (buffer-local-value 'pi-coding-agent--input-buffer chat-buf))))
+    (cond
+     ;; No session at all
+     ((null chat-buf)
+      (user-error "No pi session for this project"))
+     ;; Session visible in current frame: hide it
+     ((or (get-buffer-window-list chat-buf nil)
+          (and input-buf (get-buffer-window-list input-buf nil)))
+      (with-current-buffer chat-buf
+        (pi-coding-agent--hide-session-windows)))
+     ;; Session hidden: show it
+     (t
+      (pi-coding-agent--display-buffers chat-buf input-buf)))))
 
 ;;;; Performance Optimizations
 
